"""Calculate the cost of a Gemini API call."""


def calculate_cost(
    input_tokens: int | float | None, output_tokens: int | float | None, model: str
) -> float | None:
    """Calculate the cost of a Gemini API call."""
    return None
